/**
 * \file
 *
 * \brief Empty user application template
 *
 */

/**
 * \mainpage User Application template doxygen documentation
 *
 * \par Empty user application template
 *
 * This is a bare minimum user application template.
 *
 * For documentation of the board, go \ref group_common_boards "here" for a link
 * to the board-specific documentation.
 *
 * \par Content
 *
 * -# Include the ASF header files (through asf.h)
 * -# Minimal main function that starts with a call to board_init()
 * -# Basic usage of on-board LED and button
 * -# "Insert application code here" comment
 *
 */

/*
 * Include header files for all drivers that have been imported from
 * Atmel Software Framework (ASF).
 */
/*
 * Support and FAQ: visit <a href="http://www.atmel.com/design-support/">Atmel Support</a>
 */
#include <asf.h>

#define PWM1L_DAC IOPORT_CREATE_PIN(PIOA, 20)

#define PWM_CLOCKSOURCE_FREQ 10000000
#define PWM_FREQ 100000
#define PWM_PERIOD_TICKS PWM_CLOCKSOURCE_FREQ/PWM_FREQ
#define MEASURED_VHIGH 3
#define MEASURED_VLOW 0.1
#define MEASURED_VCC 3.28

float DAC_voltage = 1;

pwm_channel_t PWM1_config;

int main (void)
{
	//clock configuration and initialization
	sysclk_init();
	
	/*Disable the watchdog timer and configure/initialize
	port pins connected to various components incorporated 
	into the SAM4S Xplained development platform, e.g., the 
	NAND flash, the OLED interface, the LEDs, the SW0 pushbutton.*/  
	board_init();
	
	//connect peripheral B to pin A20
	pio_configure_pin(PWM1L_DAC, PIO_TYPE_PIO_PERIPH_B);

	//enable the peripheral clock for the PWM hardware
	pmc_enable_periph_clk(ID_PWM);

	//disable the PWM channel until it is properly configured
	pwm_channel_disable(PWM, PWM_CHANNEL_1);

	//PWM clock configuration
	pwm_clock_t PWMDAC_clock_config = 
	{
		.ul_clka = PWM_CLOCKSOURCE_FREQ,
		.ul_clkb = 0,
		.ul_mck = sysclk_get_cpu_hz()
	};
	
	//apply the clock configuration
	pwm_init(PWM, &PWMDAC_clock_config);
	
	//select channel 1
	PWM1_config.channel = PWM_CHANNEL_1;
	//select clock A
	PWM1_config.ul_prescaler = PWM_CMR_CPRE_CLKA;
	/*The low-pass filter is connected to the inverted output
	for PWM channel 1, so to get an active-high signal we 
	invert again by setting the polarity to low.*/
	PWM1_config.polarity = PWM_LOW;
	//left-aligned mode
	PWM1_config.alignment = PWM_ALIGN_LEFT;
	
	PWM1_config.ul_period = PWM_PERIOD_TICKS;
	//PWM1_config.ul_duty = (DAC_voltage * MEASURED_VCC) / PWM_PERIOD_TICKS;
	PWM1_config.ul_duty = ((DAC_voltage - MEASURED_VLOW) / (MEASURED_VHIGH - MEASURED_VLOW)) * PWM_PERIOD_TICKS;
	
	//apply the channel configuration
	pwm_channel_init(PWM, &PWM1_config);
		
	//configuration is complete, so enable the channels
	pwm_channel_enable(PWM, PWM_CHANNEL_1);
	
	while(1);
}
